from typing import List

from pyrman.models import Package, Release
from pyrman.pypi import parsers
from pyrman.pypi.session import PyPISession

PYPI_DEFAULT_URL = "https://pypi.org"


class PyPIClient:
    def __init__(self, pypi_url: str = PYPI_DEFAULT_URL):
        self.session = PyPISession(pypi_url)

    def authenticate(self, username: str, password: str):
        self.session.authenticate(username, password)

    def set_credentials(self, username: str, password: str):
        self.session.set_credentials(username, password)

    def get_packages(self) -> List[Package]:
        response = self.session.get("/manage/projects/")
        response.raise_for_status()
        return parsers.parse_packages(response.text)

    def get_package_management_token(self, package_name: str):
        response = self.session.get(f"/manage/project/{package_name}/releases/")
        response.raise_for_status()
        return parsers.parse_csrf_token(response.text)

    def remove_release(self, package_name: str, release_version: str):
        management_csrf_token = self.get_package_management_token(package_name)
        response = self.session.post(
            f"/manage/project/{package_name}/release/{release_version}/",
            data={"confirm_delete_version": release_version, "csrf_token": management_csrf_token,},
        )
        if response.status_code == 404:
            management_url = self.session.absolute_url(f"/manage/project/{package_name}/releases")
            raise ValueError(
                f"Release {release_version} of package {package_name} not found on PyPI. ({management_url})"
            )
        response.raise_for_status()

    def list_releases(self, package: str) -> List[Release]:
        response = self.session.get(f"/manage/project/{package}/releases/")
        response.raise_for_status()
        return parsers.parse_releases(response.text)



from typing import List

from pyrman.models import Package, Release
from pyrman.pypi.client import PYPI_DEFAULT_URL, PyPIClient

REPOSITORY_DEFAULT_URL = PYPI_DEFAULT_URL


class ReleaseManagerClient:
    def __new__(cls, repository_url: str = REPOSITORY_DEFAULT_URL):
        if "pypi.org" in repository_url:
            return PyPIClient(repository_url)
        else:
            raise ValueError("Can't guess repository type from url...")

    def authenticate(self, username: str, password: str):
        raise NotImplementedError

    def get_packages(self) -> List[Package]:
        raise NotImplementedError

    def remove_release(self, package_name: str, release_version: str):
        raise NotImplementedError

    def set_credentials(self, username: str, password: str):
        raise NotImplementedError

    def list_releases(self, package: str) -> List[Release]:
        raise NotImplementedError



import os

import click

from pyrman.client import REPOSITORY_DEFAULT_URL, ReleaseManagerClient
from pyrman.context import context
from pyrman.serialize import serialize_as_yaml
from pyrman.setupmeta import get_setup_metadata


@click.group
@click.option(
    "-u", "--username", default=os.getenv("PYRMAN_USERNAME"), help="Python repository username",
)
@click.option(
    "-p", "--password", default=os.getenv("PYRMAN_PASSWORD"), help="Read python repository password from stdin",
)
@click.option(
    "--url", default=REPOSITORY_DEFAULT_URL, help="Python repository URL (defaults to https://pypi.org)",
)
def cli(username: str, password: str, url: str = None):
    """Python releases manager
    
    Manage python published release for packages that you owns on PyPI
    """
    context.client = ReleaseManagerClient(url)
    if username or password:
        context.client.set_credentials(username, password)


@cli.group()
def setup_meta():
    """Commands related to reliably parsing setup.py"""


@setup_meta.command
@click.argument("metadata_name")
def get(metadata_name: str):
    """Get value of metadata passed as argument to setup() in setup.py"""
    print(get_setup_metadata(metadata_name))


@cli.group
def package():
    """Manage packages"""


@package.command
def ls():
    """List owned packages"""
    packages = context.client.get_packages()
    print(serialize_as_yaml(packages))


@cli.group
def release():
    """Manage releases"""


@release.command
@click.argument("package")
def ls(package: str):
    releases = context.client.list_releases(package)
    print(serialize_as_yaml(releases))


@release.command
@click.argument("package")
@click.argument("version")
def rm(package: str, version: str):
    """Remove release"""
    context.client.remove_release(package, version)



from datetime import datetime
from typing import List

from bs4 import BeautifulSoup

from pyrman.models import Package, Release


def parse_datetime(d: str):
    return datetime.strptime(d, "%Y-%m-%dT%H:%M:%S+%f")


def parse_packages(html: str) -> List[Package]:
    soup = BeautifulSoup(html, "html.parser")
    package_snippets = soup.select(".package-snippet")
    packages = []
    for package_snippet in package_snippets:
        packages.append(
            Package(
                title=package_snippet.select_one(".package-snippet__title").text.split("\n")[0],
                description=package_snippet.select_one(".package-snippet__description").text.strip(),
                last_released=parse_datetime(package_snippet.select_one("time[datetime]").get("datetime")),
                package_url=package_snippet.select_one(".button:not(.button--primary)").get("href"),
                releases_url=package_snippet.select_one(".button.button--primary").get("href"),
            )
        )
    return packages


def parse_csrf_token(html: str) -> str:
    soup = BeautifulSoup(html, "html.parser")
    csrf_input = soup.select_one("input[name=csrf_token]")
    return csrf_input.get("value")


def parse_releases(html: str) -> List[Release]:
    soup = BeautifulSoup(html, "html.parser")
    release_rows = soup.select(".table--releases tr")

    releases = []
    for row in release_rows:
        try:
            releases.append(
                Release(
                    version=row.select_one("th").text.strip(),
                    release_date=parse_datetime(row.select_one("time").get("datetime")),
                )
            )
        except AttributeError:
            pass
    return releases
